@@ -5,16 +5,20 @@ from esphome.automation import maybe_simple_id, Condition
 from esphome.components import mqtt
 from esphome.const import (
     CONF_ID,
-    CONF_INTERNAL,
     CONF_DEVICE_CLASS,
     CONF_STATE,
     CONF_POSITION,
+    CONF_POSITION_COMMAND_TOPIC,
+    CONF_POSITION_STATE_TOPIC,
     CONF_TILT,
+    CONF_TILT_COMMAND_TOPIC,
+    CONF_TILT_STATE_TOPIC,
     CONF_STOP,
     CONF_MQTT_ID,
-    CONF_NAME,
+    CONF_TRIGGER_ID,
 )
 from esphome.core import CORE, coroutine_with_priority
+from esphome.cpp_helpers import setup_entity
 
 IS_PLATFORM_COMPONENT = True
 
@@ -35,7 +39,7 @@ DEVICE_CLASSES = [
 
 cover_ns = cg.esphome_ns.namespace("cover")
 
-Cover = cover_ns.class_("Cover", cg.Nameable)
+Cover = cover_ns.class_("Cover", cg.EntityBase)
 
 COVER_OPEN = cover_ns.COVER_OPEN
 COVER_CLOSED = cover_ns.COVER_CLOSED
@@ -58,32 +62,84 @@ validate_cover_operation = cv.enum(COVER_OPERATIONS, upper=True)
 OpenAction = cover_ns.class_("OpenAction", automation.Action)
 CloseAction = cover_ns.class_("CloseAction", automation.Action)
 StopAction = cover_ns.class_("StopAction", automation.Action)
+ToggleAction = cover_ns.class_("ToggleAction", automation.Action)
 ControlAction = cover_ns.class_("ControlAction", automation.Action)
 CoverPublishAction = cover_ns.class_("CoverPublishAction", automation.Action)
 CoverIsOpenCondition = cover_ns.class_("CoverIsOpenCondition", Condition)
 CoverIsClosedCondition = cover_ns.class_("CoverIsClosedCondition", Condition)
 
-COVER_SCHEMA = cv.MQTT_COMMAND_COMPONENT_SCHEMA.extend(
+# Triggers
+CoverOpenTrigger = cover_ns.class_("CoverOpenTrigger", automation.Trigger.template())
+CoverClosedTrigger = cover_ns.class_(
+    "CoverClosedTrigger", automation.Trigger.template()
+)
+
+CONF_ON_OPEN = "on_open"
+CONF_ON_CLOSED = "on_closed"
+
+COVER_SCHEMA = cv.ENTITY_BASE_SCHEMA.extend(cv.MQTT_COMMAND_COMPONENT_SCHEMA).extend(
     {
         cv.GenerateID(): cv.declare_id(Cover),
         cv.OnlyWith(CONF_MQTT_ID, "mqtt"): cv.declare_id(mqtt.MQTTCoverComponent),
         cv.Optional(CONF_DEVICE_CLASS): cv.one_of(*DEVICE_CLASSES, lower=True),
-        # TODO: MQTT topic options
+        cv.Optional(CONF_POSITION_COMMAND_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.subscribe_topic
+        ),
+        cv.Optional(CONF_POSITION_STATE_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.subscribe_topic
+        ),
+        cv.Optional(CONF_TILT_COMMAND_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.subscribe_topic
+        ),
+        cv.Optional(CONF_TILT_STATE_TOPIC): cv.All(
+            cv.requires_component("mqtt"), cv.subscribe_topic
+        ),
+        cv.Optional(CONF_ON_OPEN): automation.validate_automation(
+            {
+                cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(CoverOpenTrigger),
+            }
+        ),
+        cv.Optional(CONF_ON_CLOSED): automation.validate_automation(
+            {
+                cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(CoverClosedTrigger),
+            }
+        ),
     }
 )
 
 
 async def setup_cover_core_(var, config):
-    cg.add(var.set_name(config[CONF_NAME]))
-    if CONF_INTERNAL in config:
-        cg.add(var.set_internal(config[CONF_INTERNAL]))
+    await setup_entity(var, config)
+
     if CONF_DEVICE_CLASS in config:
         cg.add(var.set_device_class(config[CONF_DEVICE_CLASS]))
 
+    for conf in config.get(CONF_ON_OPEN, []):
+        trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
+        await automation.build_automation(trigger, [], conf)
+    for conf in config.get(CONF_ON_CLOSED, []):
+        trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
+        await automation.build_automation(trigger, [], conf)
+
     if CONF_MQTT_ID in config:
         mqtt_ = cg.new_Pvariable(config[CONF_MQTT_ID], var)
         await mqtt.register_mqtt_component(mqtt_, config)
 
+        if CONF_POSITION_STATE_TOPIC in config:
+            cg.add(
+                mqtt_.set_custom_position_state_topic(config[CONF_POSITION_STATE_TOPIC])
+            )
+        if CONF_POSITION_COMMAND_TOPIC in config:
+            cg.add(
+                mqtt_.set_custom_position_command_topic(
+                    config[CONF_POSITION_COMMAND_TOPIC]
+                )
+            )
+        if CONF_TILT_STATE_TOPIC in config:
+            cg.add(mqtt_.set_custom_tilt_state_topic(config[CONF_TILT_STATE_TOPIC]))
+        if CONF_TILT_COMMAND_TOPIC in config:
+            cg.add(mqtt_.set_custom_tilt_command_topic(config[CONF_TILT_COMMAND_TOPIC]))
+
 
 async def register_cover(var, config):
     if not CORE.has_id(config[CONF_ID]):
@@ -117,6 +173,12 @@ async def cover_stop_to_code(config, action_id, template_arg, args):
     return cg.new_Pvariable(action_id, template_arg, paren)
 
 
+@automation.register_action("cover.toggle", ToggleAction, COVER_ACTION_SCHEMA)
+def cover_toggle_to_code(config, action_id, template_arg, args):
+    paren = yield cg.get_variable(config[CONF_ID])
+    yield cg.new_Pvariable(action_id, template_arg, paren)
+
+
 COVER_CONTROL_ACTION_SCHEMA = cv.Schema(
     {
         cv.Required(CONF_ID): cv.use_id(Cover),
