@@ -1,5 +1,10 @@
 #include "scd30.h"
 #include "esphome/core/log.h"
+#include "esphome/core/hal.h"
+
+#ifdef USE_ESP8266
+#include <Wire.h>
+#endif
 
 namespace esphome {
 namespace scd30 {
@@ -23,7 +28,7 @@ static const uint16_t SCD30_CMD_SOFT_RESET = 0xD304;
 void SCD30Component::setup() {
   ESP_LOGCONFIG(TAG, "Setting up scd30...");
 
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
   Wire.setClockStretchLimit(150000);
 #endif
 
@@ -51,11 +56,11 @@ void SCD30Component::setup() {
       return;
     }
   }
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
   // According ESP32 clock stretching is typically 30ms and up to 150ms "due to
   // internal calibration processes". The I2C peripheral only supports 13ms (at
   // least when running at 80MHz).
-  // In practise it seems that clock stretching occurs during this calibration
+  // In practice it seems that clock stretching occurs during this calibration
   // calls. It also seems that delays in between calls makes them
   // disappear/shorter. Hence work around with delays for ESP32.
   //
@@ -64,6 +69,16 @@ void SCD30Component::setup() {
   delay(30);
 #endif
 
+  if (!this->write_command_(SCD30_CMD_MEASUREMENT_INTERVAL, update_interval_)) {
+    ESP_LOGE(TAG, "Sensor SCD30 error setting update interval.");
+    this->error_code_ = MEASUREMENT_INIT_FAILED;
+    this->mark_failed();
+    return;
+  }
+#ifdef USE_ESP32
+  delay(30);
+#endif
+
   // The start measurement command disables the altitude compensation, if any, so we only set it if it's turned on
   if (this->altitude_compensation_ != 0xFFFF) {
     if (!this->write_command_(SCD30_CMD_ALTITUDE_COMPENSATION, altitude_compensation_)) {
@@ -73,7 +88,7 @@ void SCD30Component::setup() {
       return;
     }
   }
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
   delay(30);
 #endif
 
@@ -83,7 +98,7 @@ void SCD30Component::setup() {
     this->mark_failed();
     return;
   }
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
   delay(30);
 #endif
 
@@ -94,6 +109,12 @@ void SCD30Component::setup() {
     this->mark_failed();
     return;
   }
+
+  // check each 500ms if data is ready, and read it in that case
+  this->set_interval("status-check", 500, [this]() {
+    if (this->is_data_ready_())
+      this->update();
+  });
 }
 
 void SCD30Component::dump_config() {
@@ -123,19 +144,13 @@ void SCD30Component::dump_config() {
   ESP_LOGCONFIG(TAG, "  Automatic self calibration: %s", ONOFF(this->enable_asc_));
   ESP_LOGCONFIG(TAG, "  Ambient pressure compensation: %dmBar", this->ambient_pressure_compensation_);
   ESP_LOGCONFIG(TAG, "  Temperature offset: %.2f °C", this->temperature_offset_);
-  LOG_UPDATE_INTERVAL(this);
+  ESP_LOGCONFIG(TAG, "  Update interval: %ds", this->update_interval_);
   LOG_SENSOR("  ", "CO2", this->co2_sensor_);
   LOG_SENSOR("  ", "Temperature", this->temperature_sensor_);
   LOG_SENSOR("  ", "Humidity", this->humidity_sensor_);
 }
 
 void SCD30Component::update() {
-  /// Check if measurement is ready before reading the value
-  if (!this->write_command_(SCD30_CMD_GET_DATA_READY_STATUS)) {
-    this->status_set_warning();
-    return;
-  }
-
   uint16_t raw_read_status[1];
   if (!this->read_data_(raw_read_status, 1) || raw_read_status[0] == 0x00) {
     this->status_set_warning();
@@ -181,6 +196,17 @@ void SCD30Component::update() {
   });
 }
 
+bool SCD30Component::is_data_ready_() {
+  if (!this->write_command_(SCD30_CMD_GET_DATA_READY_STATUS)) {
+    return false;
+  }
+  uint16_t is_data_ready;
+  if (!this->read_data_(&is_data_ready, 1)) {
+    return false;
+  }
+  return is_data_ready == 1;
+}
+
 bool SCD30Component::write_command_(uint16_t command) {
   // Warning ugly, trick the I2Ccomponent base by setting register to the first 8 bit.
   return this->write_byte(command >> 8, command & 0xFF);
@@ -193,7 +219,7 @@ bool SCD30Component::write_command_(uint16_t command, uint16_t data) {
   raw[2] = data >> 8;
   raw[3] = data & 0xFF;
   raw[4] = sht_crc_(raw[2], raw[3]);
-  return this->write_bytes_raw(raw, 5);
+  return this->write(raw, 5) == i2c::ERROR_OK;
 }
 
 uint8_t SCD30Component::sht_crc_(uint8_t data1, uint8_t data2) {
@@ -221,10 +247,9 @@ uint8_t SCD30Component::sht_crc_(uint8_t data1, uint8_t data2) {
 
 bool SCD30Component::read_data_(uint16_t *data, uint8_t len) {
   const uint8_t num_bytes = len * 3;
-  auto *buf = new uint8_t[num_bytes];
+  std::vector<uint8_t> buf(num_bytes);
 
-  if (!this->parent_->raw_receive(this->address_, buf, num_bytes)) {
-    delete[](buf);
+  if (this->read(buf.data(), num_bytes) != i2c::ERROR_OK) {
     return false;
   }
 
@@ -233,13 +258,11 @@ bool SCD30Component::read_data_(uint16_t *data, uint8_t len) {
     uint8_t crc = sht_crc_(buf[j], buf[j + 1]);
     if (crc != buf[j + 2]) {
       ESP_LOGE(TAG, "CRC8 Checksum invalid! 0x%02X != 0x%02X", buf[j + 2], crc);
-      delete[](buf);
       return false;
     }
     data[i] = (buf[j] << 8) | buf[j + 1];
   }
 
-  delete[](buf);
   return true;
 }
 
