@@ -1,10 +1,13 @@
 #include "sntp_component.h"
 #include "esphome/core/log.h"
 
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
 #include "lwip/apps/sntp.h"
+#ifdef USE_ESP_IDF
+#include "esp_sntp.h"
 #endif
-#ifdef ARDUINO_ARCH_ESP8266
+#endif
+#ifdef USE_ESP8266
 #include "sntp.h"
 #endif
 
@@ -20,13 +23,13 @@ static const char *const TAG = "sntp";
 
 void SNTPComponent::setup() {
   ESP_LOGCONFIG(TAG, "Setting up SNTP...");
-#ifdef ARDUINO_ARCH_ESP32
+#ifdef USE_ESP32
   if (sntp_enabled()) {
     sntp_stop();
   }
   sntp_setoperatingmode(SNTP_OPMODE_POLL);
 #endif
-#ifdef ARDUINO_ARCH_ESP8266
+#ifdef USE_ESP8266
   sntp_stop();
 #endif
 
@@ -37,6 +40,9 @@ void SNTPComponent::setup() {
   if (!this->server_3_.empty()) {
     sntp_setservername(2, strdup(this->server_3_.c_str()));
   }
+#ifdef USE_ESP_IDF
+  sntp_set_sync_interval(this->get_update_interval());
+#endif
 
   sntp_init();
 }
@@ -47,7 +53,16 @@ void SNTPComponent::dump_config() {
   ESP_LOGCONFIG(TAG, "  Server 3: '%s'", this->server_3_.c_str());
   ESP_LOGCONFIG(TAG, "  Timezone: '%s'", this->timezone_.c_str());
 }
-void SNTPComponent::update() {}
+void SNTPComponent::update() {
+#ifndef USE_ESP_IDF
+  // force resync
+  if (sntp_enabled()) {
+    sntp_stop();
+    this->has_time_ = false;
+    sntp_init();
+  }
+#endif
+}
 void SNTPComponent::loop() {
   if (this->has_time_)
     return;
@@ -56,9 +71,8 @@ void SNTPComponent::loop() {
   if (!time.is_valid())
     return;
 
-  char buf[128];
-  time.strftime(buf, sizeof(buf), "%c");
-  ESP_LOGD(TAG, "Synchronized time: %s", buf);
+  ESP_LOGD(TAG, "Synchronized time: %d-%d-%d %d:%02d:%02d", time.year, time.month, time.day_of_month, time.hour,
+           time.minute, time.second);
   this->time_sync_callback_.call();
   this->has_time_ = true;
 }
