@@ -1,6 +1,7 @@
 #pragma once
 
 #include "esphome/core/component.h"
+#include "esphome/core/hal.h"
 #include "esphome/components/sensor/sensor.h"
 
 namespace esphome {
@@ -14,7 +15,7 @@ struct Tx20ComponentStore {
   volatile uint32_t spent_time;
   volatile bool tx20_available;
   volatile bool pin_state;
-  ISRInternalGPIOPin *pin;
+  ISRInternalGPIOPin pin;
 
   void reset();
   static void gpio_intr(Tx20ComponentStore *arg);
@@ -26,7 +27,7 @@ class Tx20Component : public Component {
   /// Get the textual representation of the wind direction ('N', 'SSE', ..).
   std::string get_wind_cardinal_direction() const;
 
-  void set_pin(GPIOPin *pin) { pin_ = pin; }
+  void set_pin(InternalGPIOPin *pin) { pin_ = pin; }
   void set_wind_speed_sensor(sensor::Sensor *wind_speed_sensor) { wind_speed_sensor_ = wind_speed_sensor; }
   void set_wind_direction_degrees_sensor(sensor::Sensor *wind_direction_degrees_sensor) {
     wind_direction_degrees_sensor_ = wind_direction_degrees_sensor;
@@ -41,7 +42,7 @@ class Tx20Component : public Component {
   void decode_and_publish_();
 
   std::string wind_cardinal_direction_;
-  GPIOPin *pin_;
+  InternalGPIOPin *pin_;
   sensor::Sensor *wind_speed_sensor_;
   sensor::Sensor *wind_direction_degrees_sensor_;
   Tx20ComponentStore store_;
